@@ -209,7 +209,6 @@ app_heatmap <- function(tomic) {
             value_var = input$measurement_var,
             cluster_dim = "both",
             change_threshold = thresholded_val(),
-            plot_type = "grob",
             # suppress feature aggregatin when feature facets are present
             max_display_features = ifelse(
               is.null(input$feature_facets),
@@ -269,11 +268,12 @@ app_heatmap <- function(tomic) {
 #' @inheritParams hclust_order
 #' @param change_threshold values with a more extreme absolute change will be
 #'   thresholded to this value.
-#' @param plot_type plotly (for interactivity) or grob (for a static ggplot)
 #' @inheritParams downsample_heatmap
-#' @param x_label label for x-axis (if NULL then use \code{feature_var})
-#' @param y_label label for y-axis (if NULL then use \code{sample_var})
-#' @param colorbar_label label for color-bar; default is log2 abundance
+#' @param x_title label for x-axis (if NULL then use \code{feature_var})
+#' @param y_title label for y-axis (if NULL then use \code{sample_var})
+#' @param colorbar_title label for color-bar; default is log2 abundance
+#' @param transpose if TRUE then samples will be rows and features will be columns.
+#'   Set all other variables as if transpose was FALSE.
 #'
 #' @returns a ggplot2 grob
 #'
@@ -297,25 +297,25 @@ app_heatmap <- function(tomic) {
 #'   value_var = "expression",
 #'   change_threshold = 5,
 #'   cluster_dim = "rows",
-#'   plot_type = "grob",
-#'   distance_measure = "corr"
+#'   distance_measure = "corr",
+#'   transpose = FALSE
 #' )
 #' @export
 plot_heatmap <- function(
-    tomic,
-    feature_var = NULL,
-    sample_var = NULL,
-    value_var = NULL,
-    cluster_dim = "both",
-    distance_measure = "dist",
-    hclust_method = "ward.D2",
-    change_threshold = Inf,
-    plot_type = "grob",
-    max_display_features = 800,
-    x_label = NULL,
-    y_label = NULL,
-    colorbar_label = NULL
-    ) {
+  tomic,
+  feature_var = NULL,
+  sample_var = NULL,
+  value_var = NULL,
+  cluster_dim = "both",
+  distance_measure = "dist",
+  hclust_method = "ward.D2",
+  change_threshold = Inf,
+  max_display_features = 800,
+  x_title = NULL,
+  y_title = NULL,
+  colorbar_title = NULL,
+  transpose = FALSE
+  ) {
   checkmate::assertClass(tomic, "tomic")
 
   if ("NULL" %in% class(feature_var)) {
@@ -334,23 +334,23 @@ plot_heatmap <- function(
   checkmate::assertChoice(distance_measure, c("corr", "dist"))
   checkmate::assertString(hclust_method)
   checkmate::assertNumber(change_threshold, lower = 0)
-  checkmate::assertChoice(plot_type, c("plotly", "grob"))
   checkmate::assertNumber(max_display_features)
+  checkmate::assertLogical(transpose, len = 1)
 
-  if ("NULL" %in% class(x_label)) {
-    x_label <- feature_var
+  if ("NULL" %in% class(x_title)) {
+    x_title <- sample_var
   }
-  checkmate::assertMultiClass(x_label, c("character", "expression"))
+  checkmate::assertMultiClass(x_title, c("character", "expression"))
 
-  if ("NULL" %in% class(y_label)) {
-    y_label <- sample_var
+  if ("NULL" %in% class(y_title)) {
+    y_title <- feature_var
   }
-  checkmate::assertMultiClass(y_label, c("character", "expression"))
+  checkmate::assertMultiClass(y_title, c("character", "expression"))
 
-  if ("NULL" %in% class(colorbar_label)) {
-    colorbar_label <- expression(log[2] ~ abundance)
+  if ("NULL" %in% class(colorbar_title)) {
+    colorbar_title <- expression(log[2] ~ abundance)
   }
-  checkmate::assertMultiClass(colorbar_label, c("character", "expression"))
+  checkmate::assertMultiClass(colorbar_title, c("character", "expression"))
 
   # format convert tomic to tidy format if needed
 
@@ -418,62 +418,87 @@ plot_heatmap <- function(
        strip.background = element_rect(fill = "gray80")
     )
 
-  if (n_features > 200) {
+  distinct_features <- augmented_tidy_omic_data %>%
+    dplyr::distinct(ordered_featureId, feature_label)
+
+  distinct_samples <- augmented_tidy_omic_data %>%
+    dplyr::distinct(ordered_sampleId, sample_label)
+
+  if (transpose) {
+    x_features = pmin(max_display_features, n_features)
+    x_ordered_by = "ordered_featureId"
+    x_breaks <- distinct_features$ordered_featureId
+    x_labels <- distinct_features$feature_label
+
+    y_features = n_samples
+    y_ordered_by = "ordered_sampleId"
+    y_breaks <- distinct_samples$ordered_sampleId
+    y_labels <- distinct_samples$sample_label
+
+    tmp <- x_title
+    x_title <- y_title
+    y_title <- tmp
+  } else {
+    y_features = pmin(max_display_features, n_features)
+    y_ordered_by = "ordered_featureId"
+    y_breaks <- distinct_features$ordered_featureId
+    y_labels <- distinct_features$feature_label
+
+    x_features = n_samples
+    x_ordered_by = "ordered_sampleId"
+    x_breaks <- distinct_samples$ordered_sampleId
+    x_labels <- distinct_samples$sample_label
+  }
+
+  if (x_features > 200) {
+    heatmap_theme <- heatmap_theme +
+      theme(axis.text.x = element_blank())
+  } else {
+    heatmap_theme <- heatmap_theme +
+      theme(axis.text.x = element_text(
+        size = pmin(20, 60 * sqrt(1 / x_features)),
+        angle = 90,
+        hjust = 1
+      ))
+  }
+
+  if (y_features > 200) {
     heatmap_theme <- heatmap_theme +
       theme(axis.text.y = element_blank())
   } else {
     heatmap_theme <- heatmap_theme +
-      theme(axis.text.y = element_text(size = pmin(20, 60 * sqrt(1 / n_features))))
-  }
-
-  if (n_samples > 200) {
-    heatmap_theme <- heatmap_theme + theme(axis.text.x = element_blank())
-  } else {
-    heatmap_theme <- heatmap_theme + theme(axis.text.x = element_text(
-      size = pmin(20, 60 * sqrt(1 / n_samples)),
-      angle = 90,
-      hjust = 1
-    ))
+      theme(axis.text.y = element_text(size = pmin(20, 60 * sqrt(1 / y_features))))
   }
 
   heatmap_plot <- ggplot(
     augmented_tidy_omic_data,
     aes(
-      x = !!rlang::sym("ordered_sampleId"),
-      y = !!rlang::sym("ordered_featureId"),
+      x = !!rlang::sym(x_ordered_by),
+      y = !!rlang::sym(y_ordered_by),
       fill = !!rlang::sym(value_var)
     )
   ) +
     geom_raster() +
     scale_fill_gradient2(
-      colorbar_label,
+      colorbar_title,
       low = "steelblue1",
       mid = "black",
       high = "yellow",
       midpoint = 0
     ) +
     scale_x_discrete(
-      x_label,
-      breaks = augmented_tidy_omic_data$ordered_sampleId,
-      labels = augmented_tidy_omic_data$sample_label
+      x_title,
+      breaks = x_breaks,
+      labels = x_labels
     ) +
     scale_y_discrete(
-      y_label,
-      breaks = augmented_tidy_omic_data$ordered_featureId,
-      labels = augmented_tidy_omic_data$feature_label,
+      y_title,
+      breaks = y_breaks,
+      labels = y_labels,
       position = "right"
     ) +
     expand_limits(fill = c(-1 * change_threshold, change_threshold)) +
     heatmap_theme
 
-  if (plot_type == "grob") {
-    return(heatmap_plot)
-  } else if (plot_type == "plotly") {
-    suppressWarnings(
-      plotly::ggplotly(heatmap_plot) %>%
-        plotly::layout(margin = 0)
-    )
-  } else {
-    stop("undefined plotting type logic")
-  }
+  return(heatmap_plot)
 }
